@@ -572,8 +572,8 @@ paf24_read_s (SF_PRIVATE *psf, short *ptr, sf_count_t len)
 		return 0 ;
 	ppaf24 = (PAF24_PRIVATE*) psf->fdata ;
 
-	iptr = (int*) psf->buffer ;
-	bufferlen = sizeof (psf->buffer) / sizeof (int) ;
+	iptr = psf->u.ibuf ;
+	bufferlen = ARRAY_LEN (psf->u.ibuf) ;
 	while (len > 0)
 	{	readcount = (len >= bufferlen) ? bufferlen : len ;
 		count = paf24_read (psf, ppaf24, iptr, readcount) ;
@@ -613,8 +613,8 @@ paf24_read_f (SF_PRIVATE *psf, float *ptr, sf_count_t len)
 
 	normfact = (psf->norm_float == SF_TRUE) ? (1.0 / 0x80000000) : (1.0 / 0x100) ;
 
-	iptr = (int*) psf->buffer ;
-	bufferlen = sizeof (psf->buffer) / sizeof (int) ;
+	iptr = psf->u.ibuf ;
+	bufferlen = ARRAY_LEN (psf->u.ibuf) ;
 	while (len > 0)
 	{	readcount = (len >= bufferlen) ? bufferlen : len ;
 		count = paf24_read (psf, ppaf24, iptr, readcount) ;
@@ -640,8 +640,8 @@ paf24_read_d (SF_PRIVATE *psf, double *ptr, sf_count_t len)
 
 	normfact = (psf->norm_double == SF_TRUE) ? (1.0 / 0x80000000) : (1.0 / 0x100) ;
 
-	iptr = (int*) psf->buffer ;
-	bufferlen = sizeof (psf->buffer) / sizeof (int) ;
+	iptr = psf->u.ibuf ;
+	bufferlen = ARRAY_LEN (psf->u.ibuf) ;
 	while (len > 0)
 	{	readcount = (len >= bufferlen) ? bufferlen : len ;
 		count = paf24_read (psf, ppaf24, iptr, readcount) ;
@@ -738,8 +738,8 @@ paf24_write_s (SF_PRIVATE *psf, short *ptr, sf_count_t len)
 		return 0 ;
 	ppaf24 = (PAF24_PRIVATE*) psf->fdata ;
 
-	iptr = (int*) psf->buffer ;
-	bufferlen = sizeof (psf->buffer) / sizeof (int) ;
+	iptr = psf->u.ibuf ;
+	bufferlen = ARRAY_LEN (psf->u.ibuf) ;
 	while (len > 0)
 	{	writecount = (len >= bufferlen) ? bufferlen : len ;
 		for (k = 0 ; k < writecount ; k++)
@@ -791,8 +791,8 @@ paf24_write_f (SF_PRIVATE *psf, float *ptr, sf_count_t len)
 
 	normfact = (psf->norm_float == SF_TRUE) ? (1.0 * 0x7FFFFFFF) : (1.0 / 0x100) ;
 
-	iptr = (int*) psf->buffer ;
-	bufferlen = sizeof (psf->buffer) / sizeof (int) ;
+	iptr = psf->u.ibuf ;
+	bufferlen = ARRAY_LEN (psf->u.ibuf) ;
 	while (len > 0)
 	{	writecount = (len >= bufferlen) ? bufferlen : len ;
 		for (k = 0 ; k < writecount ; k++)
@@ -821,8 +821,8 @@ paf24_write_d (SF_PRIVATE *psf, double *ptr, sf_count_t len)
 
 	normfact = (psf->norm_double == SF_TRUE) ? (1.0 * 0x7FFFFFFF) : (1.0 / 0x100) ;
 
-	iptr = (int*) psf->buffer ;
-	bufferlen = sizeof (psf->buffer) / sizeof (int) ;
+	iptr = psf->u.ibuf ;
+	bufferlen = ARRAY_LEN (psf->u.ibuf) ;
 	while (len > 0)
 	{	writecount = (len >= bufferlen) ? bufferlen : len ;
 		for (k = 0 ; k < writecount ; k++)
