@@ -86,7 +86,7 @@ enum
 	HAVE_fact	= 0x08,
 	HAVE_PEAK	= 0x10,
 	HAVE_data	= 0x20,
-	HAVE_other  = 0x80000000
+	HAVE_other	= 0x80000000
 } ;
 
 
@@ -412,14 +412,14 @@ wav_read_header	 (SF_PRIVATE *psf, int *blockalign, int *framesperblock)
 					psf_log_printf (psf, "  time stamp : %d\n", psf->pchunk->timestamp) ;
 					psf_log_printf (psf, "    Ch   Position       Value\n") ;
 
-					cptr = (char *) psf->buffer ;
+					cptr = psf->u.cbuf ;
 					for (dword = 0 ; dword < psf->sf.channels ; dword++)
 					{	psf_binheader_readf (psf, "ef4", & (psf->pchunk->peaks [dword].value),
 														& (psf->pchunk->peaks [dword].position)) ;
 
-						LSF_SNPRINTF (cptr, sizeof (psf->buffer), "    %2d   %-12d   %g\n",
+						LSF_SNPRINTF (cptr, sizeof (psf->u.cbuf), "    %2d   %-12d   %g\n",
 								dword, psf->pchunk->peaks [dword].position, psf->pchunk->peaks [dword].value) ;
-						cptr [sizeof (psf->buffer) - 1] = 0 ;
+						cptr [sizeof (psf->u.cbuf) - 1] = 0 ;
 						psf_log_printf (psf, cptr) ;
 						} ;
 
@@ -1123,12 +1123,12 @@ wav_subchunk_parse (SF_PRIVATE *psf, int chunk)
 			case ISRC_MARKER :
 					bytesread += psf_binheader_readf (psf, "e4", &dword) ;
 					dword += (dword & 1) ;
-					if (dword > SIGNED_SIZEOF (psf->buffer))
+					if (dword > SIGNED_SIZEOF (psf->u.cbuf))
 					{	psf_log_printf (psf, "  *** %M : %d (too big)\n", chunk, dword) ;
 						return SFE_INTERNAL ;
 						} ;
 
-					cptr = (char*) psf->buffer ;
+					cptr = psf->u.cbuf ;
 					psf_binheader_readf (psf, "b", cptr, dword) ;
 					bytesread += dword ;
 					cptr [dword - 1] = 0 ;
@@ -1141,12 +1141,12 @@ wav_subchunk_parse (SF_PRIVATE *psf, int chunk)
 						bytesread += psf_binheader_readf (psf, "e44", &dword, &mark_id) ;
 						dword -= 4 ;
 						dword += (dword & 1) ;
-						if (dword > SIGNED_SIZEOF (psf->buffer))
+						if (dword > SIGNED_SIZEOF (psf->u.cbuf))
 						{	psf_log_printf (psf, "  *** %M : %d (too big)\n", chunk, dword) ;
 							return SFE_INTERNAL ;
 							} ;
 
-						cptr = (char*) psf->buffer ;
+						cptr = psf->u.cbuf ;
 						psf_binheader_readf (psf, "b", cptr, dword) ;
 						bytesread += dword ;
 						cptr [dword - 1] = 0 ;
@@ -1179,22 +1179,22 @@ wav_subchunk_parse (SF_PRIVATE *psf, int chunk)
 
 		switch (chunk)
 		{	case ISFT_MARKER :
-					psf_store_string (psf, SF_STR_SOFTWARE, (char*) psf->buffer) ;
+					psf_store_string (psf, SF_STR_SOFTWARE, psf->u.cbuf) ;
 					break ;
 			case ICOP_MARKER :
-					psf_store_string (psf, SF_STR_COPYRIGHT, (char*) psf->buffer) ;
+					psf_store_string (psf, SF_STR_COPYRIGHT, psf->u.cbuf) ;
 					break ;
 			case INAM_MARKER :
-					psf_store_string (psf, SF_STR_TITLE, (char*) psf->buffer) ;
+					psf_store_string (psf, SF_STR_TITLE, psf->u.cbuf) ;
 					break ;
 			case IART_MARKER :
-					psf_store_string (psf, SF_STR_ARTIST, (char*) psf->buffer) ;
+					psf_store_string (psf, SF_STR_ARTIST, psf->u.cbuf) ;
 					break ;
 			case ICMT_MARKER :
-					psf_store_string (psf, SF_STR_COMMENT, (char*) psf->buffer) ;
+					psf_store_string (psf, SF_STR_COMMENT, psf->u.cbuf) ;
 					break ;
 			case ICRD_MARKER :
-					psf_store_string (psf, SF_STR_DATE, (char*) psf->buffer) ;
+					psf_store_string (psf, SF_STR_DATE, psf->u.cbuf) ;
 					break ;
 			} ;
 
@@ -1231,9 +1231,9 @@ wav_read_smpl_chunk (SF_PRIVATE *psf, unsigned int chunklen)
 
 	bytesread += psf_binheader_readf (psf, "e4", &dword) ;
 	if (dword != 0)
-	{	LSF_SNPRINTF ((char*) psf->buffer, sizeof (psf->buffer), "%f",
+	{	LSF_SNPRINTF (psf->u.cbuf, sizeof (psf->u.cbuf), "%f",
 				 (1.0 * 0x80000000) / ((unsigned int) dword)) ;
-		psf_log_printf (psf, "  Pitch Fract. : %s\n", (char*) psf->buffer) ;
+		psf_log_printf (psf, "  Pitch Fract. : %s\n", psf->u.cbuf) ;
 		}
 	else
 		psf_log_printf (psf, "  Pitch Fract. : 0\n") ;
@@ -1242,9 +1242,9 @@ wav_read_smpl_chunk (SF_PRIVATE *psf, unsigned int chunklen)
 	psf_log_printf (psf, "  SMPTE Format : %u\n", dword) ;
 
 	bytesread += psf_binheader_readf (psf, "e4", &dword) ;
-	LSF_SNPRINTF ((char*) psf->buffer, sizeof (psf->buffer), "%02d:%02d:%02d %02d",
+	LSF_SNPRINTF (psf->u.cbuf, sizeof (psf->u.cbuf), "%02d:%02d:%02d %02d",
 		 (dword >> 24) & 0x7F, (dword >> 16) & 0x7F, (dword >> 8) & 0x7F, dword & 0x7F) ;
-	psf_log_printf (psf, "  SMPTE Offset : %s\n", psf->buffer) ;
+	psf_log_printf (psf, "  SMPTE Offset : %s\n", psf->u.cbuf) ;
 
 	bytesread += psf_binheader_readf (psf, "e4", &loop_count) ;
 	psf_log_printf (psf, "  Loop Count   : %u\n", loop_count) ;
@@ -1312,14 +1312,29 @@ wav_read_smpl_chunk (SF_PRIVATE *psf, unsigned int chunklen)
 ** The acid chunk goes a little something like this:
 **
 ** 4 bytes          'acid'
-** 4 bytes (int)     length of chunk
-** 4 bytes (int)     ???
+** 4 bytes (int)     length of chunk starting at next byte
+**
+** 4 bytes (int)     type of file:
+**        this appears to be a bit mask,however some combinations
+**        are probably impossible and/or qualified as "errors"
+**
+**        0x01 On: One Shot         Off: Loop
+**        0x02 On: Root note is Set Off: No root
+**        0x04 On: Stretch is On,   Off: Strech is OFF
+**        0x08 On: Disk Based       Off: Ram based
+**        0x10 On: ??????????       Off: ????????? (Acidizer puts that ON)
+**
 ** 2 bytes (short)      root note
-** 2 bytes (short)      ???
-** 4 bytes (float)      ???
-** 4 bytes (int)     number of beats
-** 2 bytes (short)      meter denominator
-** 2 bytes (short)      meter numerator
+**        if type 0x10 is OFF : [C,C#,(...),B] -> [0x30 to 0x3B]
+**        if type 0x10 is ON  : [C,C#,(...),B] -> [0x3C to 0x47]
+**         (both types fit on same MIDI pitch albeit different octaves, so who cares)
+**
+** 2 bytes (short)      ??? always set to 0x8000
+** 4 bytes (float)      ??? seems to be always 0
+** 4 bytes (int)        number of beats
+** 2 bytes (short)      meter denominator   //always 4 in SF/ACID
+** 2 bytes (short)      meter numerator     //always 4 in SF/ACID
+**                      //are we sure about the order?? usually its num/denom
 ** 4 bytes (float)      tempo
 **
 */
@@ -1327,26 +1342,46 @@ wav_read_smpl_chunk (SF_PRIVATE *psf, unsigned int chunklen)
 static int
 wav_read_acid_chunk (SF_PRIVATE *psf, unsigned int chunklen)
 {	unsigned int bytesread = 0 ;
-	int	beats ;
+	int	beats, flags ;
 	short rootnote, q1, meter_denom, meter_numer ;
-	float q2, q3, tempo ;
+	float q2, tempo ;
 
 	chunklen += (chunklen & 1) ;
 
-	bytesread += psf_binheader_readf (psf, "e224f", &rootnote, &q1, &q2, &q3) ;
-	LSF_SNPRINTF ((char*) psf->buffer, sizeof (psf->buffer), "%f", q2) ;
-	psf_log_printf (psf, "  Root note : %d\n  ????      : %d\n  ????      : %s\n  ????      : %d\n",
-				rootnote, q1, psf->buffer, q3) ;
+	bytesread += psf_binheader_readf (psf, "e422f", &flags, &rootnote, &q1, &q2) ;
+
+	LSF_SNPRINTF (psf->u.cbuf, sizeof (psf->u.cbuf), "%f", q2) ;
+
+	psf_log_printf (psf, "  Flags     : 0x%04x (%s,%s,%s,%s,%s)\n", flags,
+			(flags & 0x01) ? "OneShot" : "Loop",
+			(flags & 0x02) ? "RootNoteValid" : "RootNoteInvalid",
+			(flags & 0x04) ? "StretchOn" : "StretchOff",
+			(flags & 0x08) ? "DiskBased" : "RAMBased",
+			(flags & 0x10) ? "??On" : "??Off") ;
+
+	psf_log_printf (psf, "  Root note : 0x%x\n  ????      : 0x%04x\n  ????      : %s\n",
+				rootnote, q1, psf->u.cbuf) ;
 
 	bytesread += psf_binheader_readf (psf, "e422f", &beats, &meter_denom, &meter_numer, &tempo) ;
-	LSF_SNPRINTF ((char*) psf->buffer, sizeof (psf->buffer), "%f", tempo) ;
+	LSF_SNPRINTF (psf->u.cbuf, sizeof (psf->u.cbuf), "%f", tempo) ;
 	psf_log_printf (psf, "  Beats     : %d\n  Meter     : %d/%d\n  Tempo     : %s\n",
-				beats, meter_denom, meter_numer, psf->buffer) ;
+				beats, meter_numer, meter_denom, psf->u.cbuf) ;
 
 	psf_binheader_readf (psf, "j", chunklen - bytesread) ;
 
+	if ((psf->loop_info = calloc (1, sizeof (SF_LOOP_INFO))) == NULL)
+		return SFE_MALLOC_FAILED ;
+
+	psf->loop_info->time_sig_num	= meter_numer ;
+	psf->loop_info->time_sig_den	= meter_denom ;
+	psf->loop_info->loop_mode		= (flags & 0x01) ? SF_LOOP_NONE : SF_LOOP_FORWARD ;
+	psf->loop_info->num_beats		= beats ;
+	psf->loop_info->bpm				= tempo ;
+	psf->loop_info->root_key		= (flags & 0x02) ? rootnote : -1 ;
+
 	return 0 ;
 } /* wav_read_acid_chunk */
+
 /*
 ** Do not edit or modify anything in this comment block.
 ** The arch-tag line is a file identity tag for the GNU Arch
